/* bios.c - CP/M BIOS simulator
 * Hack of original for Unix - ++tlb
 * Another hack for DJGPP
 */

#include <stdio.h>
#include <fcntl.h>
#include "bios.h"

extern char key;

extern FILE *fplist;
extern FILE *fprdr;
extern FILE *fppun;

/* This module logically consists of 5 parts: phisical disk I/O, direct */
/* I/O port simulation, normal (simulated) disk I/O, character I/O, and */
/* common initialize routines.	The first 2 parts are optional, and you */
/* can eliminate the codes when they are not necessary.			*/

/* ==================================================================== */
/* PART 1: NORMAL DISK							*/
/* ==================================================================== */

/* open mode to binary read/write					*/

/* dos */
#define OMODES		( O_RDWR | O_BINARY )
/* Unix
#define OMODES		O_RDWR
*/

int		disk[NDISKS];
extern char	name[NDISKS][50];
int secnum;

/* read disk routine  - all disk access assume 0 track offset (no system) */
x_rd(ptr, drv, trk, sec)
char * ptr;
int drv, trk, sec;
{
	int ret;
	long offset;

/*
printf("drv = %d, trk = %d, sec = %d\n", drv, trk, sec);
*/

	switch (disk[drv]) {
		case EMPTY:
			return(1);
		default:
			if (trk < 0) return(1);
			if (drv == 7) {	/* drive h is used for file xfer, has 26 SPT */
				secnum = (trk * 26) + sec;
			}
			else { /* all the rest are 8MB drives - 128 SPT */
				secnum = (trk * DPB_SPT) + sec;
			}
			lseek(disk[drv], (long)secnum * SECLEN, 0);
			ret = read(disk[drv], ptr, SECLEN);
			if (ret == 128) {
				return(0);
			}
			else {
				return(1);
			}
		
	}
}

/* write disk routine - all disk accesses assume 0 track offset (no system) */
x_wr(ptr, drv, trk, sec)
char *ptr;
int drv, trk, sec;
{
	int ret;

/*
printf("WRITE: drv = %d, trk = %d, sec = %d\n", drv, trk, sec);
*/

	switch (disk[drv]) {
		case EMPTY:
			return(1);
		default:
			if (trk < 0) return(1);
			if (drv == 7) {	/* drive h is used for file xfer, has 26 SPT */
				secnum = (trk * 26) + sec;
			}
			else { /* all the rest are 8MB drives - 128 SPT */
				secnum = (trk * DPB_SPT) + sec;
			}
			lseek(disk[drv], (long)secnum * SECLEN, 0);
			ret = write(disk[drv], ptr, SECLEN);
			if (ret == 128) {
				return(0);
			}
			else {
				return(1);
			}
	}
}

/* ==================================================================== */
/* PART 4: CHARACTER I/O						*/
/* ==================================================================== */

/* pseudo BIOS functions */
x_const()
{
	return(kbhit());
}

x_conin()
{
	return(getch());
}

x_conot(c)
char c;
{
    putch(c);
}

x_lstot(c)
char c;
{
	/* send char to printer (list device) */
	fputc(c, fplist);
}

/* printer (list device) status */
x_lstst()
{
	/* we're always ready */
	return(0xff);
}

/* input from CP/M RDR: device */
x_rdrin()
{
	int c;
	if (fprdr == NULL) {
		/* without returning something, CP/M will hang waiting
		 * for input - send back ASCII EOF (^Z) */
		return(0x26);
	}
	else {
		c = fgetc(fprdr);
		if (c == EOF) {
			/* close file */
			fclose(fprdr);
			/* reset fprdr so we'll know next time */
			fprdr = NULL;
			/* return ASCII EOF */
			return(0x26);
		}
		else {
			/* 8 bit return */
			return(c);
		}
	}
}

/* output to CP/M PUN: device */
x_punot(char c)
{
	/* send char to punch device */
	fputc(c, fppun);
}

/* ==================================================================== */
/* PART 5: INITIALIZE ROUTINES						*/
/* ==================================================================== */

int cbrk;			/* old check-break-key mode */
int cimode, comode;		/* old console modes */

/* initialize disks */
x_init()
{
	int i, d, x;

	/* try and open all file names given */
	for (i = 0; i < NDISKS; i++) {
		if (name[i][0] == '\0') {
			d = EMPTY;
		}
		else {
			d = open(name[i], OMODES);
			if ( d < 0 ) {
				fprintf(stderr, "Cannot open %s\n", name[i]);
				return(1);
			}
		}
		disk[i] = d;
	}

    return(0);
}

/* exit fixup - close all open disk files */
x_exit()
{
    int i;

    for ( i = 0; i < NDISKS; i++ ) {
		if ( disk[ i ] >= 0 ) close( disk[ i ] );
    }
}

/* output char to stdout */
putch(c)
char c;
{
	write(1,&c,1);
}

